# Reseda version 1.0.0
# ResedaNode.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Accessors

# 1) Imported modules
# -------------------

from S2MLCore import *
from ResedaType import *

# 2) Creator
# ----------

class ResedaNode(S2MLPort):
  def __init__(self, path, container):
    S2MLPort.__init__(self, ResedaType.NODE, None, path, container)
    if path.IsIdentifier():
      self.SetIdentifierAndInsertInContainerDictionary(path.ToString())
    self._index = -1
    self._inEdges = []
    self._outEdges = []
    self._treated = False

# 3) Accessors
# ------------

  def GetIndex(self):
    return self._index

  def SetIndex(self, index):
    self._index = index

  def ResetInEdges(self):
    self._inEdges.clear()

  def GetInEdges(self):
    return self._inEdges

  def AddInEdge(self, edge):
    self._inEdges.append(edge)

  def ResetOutEdges(self):
    self._outEdges.clear()

  def GetOutEdges(self):
    return self._outEdges

  def AddOutEdge(self, edge):
    self._outEdges.append(edge)

  def IsTreated(self):
    return self._treated

  def SetTreated(self, value):
    self._treated = value
