# Reseda version 1.0.0
# MusetElement.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Accessors

# 1) Imported modules
# -------------------

from S2MLCore import *
from ResedaModels import *

# 2) Creator
# ----------

class MusetElement:
  NONE = 0
  CONSTANT = 1
  BLOCK = 2
  NODE = 3
  EDGE = 4
  SET = 5

  def __init__(self, type, element):
    self._type = type
    self._element = element
    self._flag = 0

  @staticmethod
  def NewNone():
    return MusetElement(MusetElement.NONE, None)

  @staticmethod
  def NewConstant(constant):
    return MusetElement(MusetElement.CONSTANT, constant)

  @staticmethod
  def NewBlock(block):
    return MusetElement(MusetElement.BLOCK, block)

  @staticmethod
  def NewNode(node):
    return MusetElement(MusetElement.NODE, node)

  @staticmethod
  def NewEdge(edge):
    return MusetElement(MusetElement.EDGE, edge)

# 3) Accessors
# ------------

  def GetType(self):
    return self._type

  def IsNone(self):
    return self._type==MusetElement.NONE

  def IsConstant(self):
    return self._type==MusetElement.CONSTANT

  def IsSymbolicConstant(self):
    if self._type!=MusetElement.CONSTANT:
      return False
    return self.IsSymbol()

  def IsBlock(self):
    return self._type==MusetElement.BLOCK

  def IsNode(self):
    return self._type==MusetElement.NODE

  def IsEdge(self):
    return self._type==MusetElement.EDGE

  def IsNetworkElement(self):
    return self.IsBlock() or self.IsNode() or self.IsEdge()

  def IsSet(self):
    return self._type==MusetElement.SET

  def IsEqual(self, other):
    if self.GetType()!=other.GetType():
      return False
    if self.IsNone():
      return True
    if self.IsConstant():
      return self._element.IsEqual(other._element)
    if self.IsBlock():
      return self._element==other._element
    if self.IsNode():
      return self._element==other._element
    if self.IsEdge():
      return self._element==other._element
    if self.IsSet():
      return self._element.IsEqual(other._element)
    return False

  def GetConstant(self):
    if not self.IsConstant():
      return None
    return self._element

  def GetBlock(self):
    if not self.IsBlock():
      return None
    return self._element

  def GetNode(self):
    if not self.IsNode():
      return None
    return self._element

  def GetEdge(self):
    if not self.IsEdge():
      return None
    return self._element

  def GetNetworkElement(self):
    if not self.IsNetworkElement():
      return None
    return self._element

  def GetSet(self):
    if not self.IsSet():
      return None
    return self

  def GetFlag(self):
    return self._flag

  def SetFlag(self, value):
    self._flag = value