# Reseda version 1.0.0
# ResedaScriptChecker.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda, a pedagogical implementation of graphs as a modeling tool.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Builtin command compute

# 1) Imported modules
# -------------------

from S2MLCore import *

# 2) Creator
# ----------

class ResedaScriptChecker(S2MLScriptChecker):
  def __init__(self, engine):
    S2MLScriptChecker.__init__(self, engine)

# 3) Builtin command compute
# --------------------------

  def CheckCommandCompute(self, command):
    if command.GetNumberOfArguments()==0:
      self.RaiseErrorInCommand(command, "Wrong number of arguments\n")
      return 1
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what=="shortest-path" or what=="dates" or what=="maximum-flow":
      return self._CheckCommandComputeGraphCalculations(command)
    if what=="values" or what=="parameters" or what=="attributes" or what=="properties":
      return self._CheckCommandComputeValues(command)
    self.RaiseErrorInCommand(command, "Unknown option {0:s}\n".format(what))
    return 1

  def _CheckCommandComputeGraphCalculations(self, command):
    error = self.CheckNumberOfArgumentsOfCommand(command, 4)
    if error!=0:
      return 1
    sourceNode = self.GetCommandArgumentAsIdentifier(command, 1)
    if sourceNode==None:
      return 1
    targetNode = self.GetCommandArgumentAsIdentifier(command, 2)
    if targetNode==None:
      return 1
    attributeName = self.GetCommandArgumentAsIdentifier(command, 3)
    if attributeName==None:
      return 1
    error = self.CheckOptions(command)
    if error!=0:
      return 1
    return 0

  def _CheckCommandComputeValues(self, command):
    error = self.CheckNumberOfArgumentsOfCommand(command, 1)
    if error!=0:
      return 1
    error = self.CheckOptions(command)
    if error!=0:
      return 1
    return 0

