# S2MLCore version 1.2.0
# S2MLToken.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Accessors

# 1) Imported modules
# -------------------

import string
import sys
import re

from S2MLType import *

# 2) Creator
# ----------

class S2MLToken:

  # Generic tokens
  TOKEN_INTEGER = 1
  TOKEN_REAL = 2
  TOKEN_IDENTIFIER = 3
  TOKEN_PATH = 4
  TOKEN_COMMENT = 5
  TOKEN_NON_TERMINATED_COMMENT = 6
  TOKEN_UNKNOWN = 7

  # Symbols
  TOKEN_LPAR = 101
  TOKEN_RPAR = 102
  TOKEN_LBRACE = 103
  TOKEN_RBRACE = 104
  TOKEN_LBRACK = 105
  TOKEN_RBRACK = 106
  TOKEN_SEMICOLON = 107
  TOKEN_COMMA = 108
  TOKEN_COLON = 109
  TOKEN_EQUAL = 110
  TOKEN_EQ = 111
  TOKEN_PLUS = 112
  TOKEN_MINUS = 113
  TOKEN_STAR = 114
  TOKEN_SLASH = 115
  TOKEN_LT = 116
  TOKEN_LEQ = 117
  TOKEN_GT = 118
  TOKEN_GEQ = 119
  TOKEN_EXCLAMATION_MARK = 121
  TOKEN_DF = 122
  TOKEN_NEG = 199

  # Keywords for S2ML constructs
  KEYWORD_DOMAIN = 201
  KEYWORD_BLOCK = 203
  KEYWORD_CLASS = 204
  KEYWORD_END = 205
  KEYWORD_CLONES = 206
  KEYWORD_AS = 207
  KEYWORD_EMBEDS = 208
  KEYWORD_EXTENDS = 209
  KEYWORD_FUNCTION = 211
  KEYWORD_OPERATOR = 213
  KEYWORD_ATTRIBUTE = 220
  KEYWORD_PARAMETER = 221
  KEYWORD_BOOLEAN = 231
  KEYWORD_INTEGER = 232
  KEYWORD_REAL = 233
  KEYWORD_SYMBOL = 234
  KEYWORD_STRING = 235
  KEYWORD_ANY = 236
  KEYWORD_MAIN = 241
  KEYWORD_OWNER = 242
  KEYWORD_SELF = 242

  # Keywords for stochastic expressions
  KEYWORD_ADD = 311
  KEYWORD_SUB = 312
  KEYWORD_MUL = 313
  KEYWORD_DIV = 314
  KEYWORD_NEG = 315
  KEYWORD_MIN = 316
  KEYWORD_MAX = 317
  KEYWORD_PI = 318
  KEYWORD_FALSE = 321
  KEYWORD_TRUE = 322
  KEYWORD_AND = 323
  KEYWORD_OR  = 324
  KEYWORD_NOT = 325
  KEYWORD_EQ  = 331
  KEYWORD_DF  = 332
  KEYWORD_LT  = 333
  KEYWORD_GT  = 334
  KEYWORD_LEQ = 335
  KEYWORD_GEQ = 336
  KEYWORD_WITHIN = 337
  KEYWORD_COUNT = 341
  KEYWORD_IF = 342
  KEYWORD_THEN = 343
  KEYWORD_ELSE = 344
  KEYWORD_UNIFORM_DEVIATE = 351
  KEYWORD_NORMAL_DEVIATE = 352
  KEYWORD_LOGNORMAL_DEVIATE = 353
  KEYWORD_TRIANGULAR_DEVIATE = 354
  KEYWORD_EXPONENTIAL_DEVIATE = 355
  KEYWORD_WEIBULL_DEVIATE = 356
  KEYWORD_PIECEWISE_UNIFORM_DEVIATE = 357
  KEYWORD_STEPWISE_DEVIATE = 358
  KEYWORD_MISSION_TIME = 361
  KEYWORD_EXP = 371
  KEYWORD_LOG = 372
  KEYWORD_POW = 373
  KEYWORD_SQRT = 374
  KEYWORD_FLOOR = 375
  KEYWORD_CEIL = 375
  KEYWORD_ABS = 376
  KEYWORD_MOD = 377
  KEYWORD_GAMMA = 378
  KEYWORD_SIN = 381
  KEYWORD_COS = 382
  KEYWORD_TAN = 383
  KEYWORD_ASIN = 384
  KEYWORD_ACOS = 385
  KEYWORD_ATAN = 386
  KEYWORD_CONSTANT_DISTRIBUTION = 391
  KEYWORD_EXPONENTIAL_DISTRIBUTION = 392
  KEYWORD_GLM_DISTRIBUTION = 393
  KEYWORD_WEIBULL_DISTRIBUTION = 394
  KEYWORD_DIRAC_DISTRIBUTION = 395
  KEYWORD_PIECEWISE_UNIFORM_DISTRIBUTION = 397
  KEYWORD_STEPWISE_DISTRIBUTION = 398

  def __init__(self, type, string, fileName, lineNumber, columnNumber):
    self.type = type
    self.string = string
    self.fileName = fileName
    self.lineNumber = lineNumber
    self.columnNumber = columnNumber

# 3) Accessors
# ------------

  @staticmethod
  def GetNameOfType(type):
    return type

  def GetType(self):
    return self.type

  def GetString(self):
    return self.string

  def GetFileName(self):
    return self.fileName

  def GetLineNumber(self):
    return self.lineNumber

  def GetColumnNumber(self):
    return self.columnNumber
