# S2MLCore version 1.2.0
# S2MLInterpreter.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Accessors
# 4) Scripts

# 1) Imported modules
# -------------------

from S2MLMatrices import *
from S2MLStatistics import *
from S2MLManagedValue import *
from S2MLEnvironmentManager import *

from S2MLScriptPrinter import *
from S2MLScriptParser import *
from S2MLScriptChecker import *

# 2) Creator
# ----------

class S2MLInterpreter:
  def __init__(self, engine):
    from S2MLEngine import S2MLEngine
    self._engine = engine
    self._scriptPrinter = None
    self._scriptParser = None
    self._scriptChecker = None
    self._currentScript = None
    self._outputFile = sys.stdout
    self._errorFile = sys.stderr

  def _RaiseError(self, message):
    self._engine.RaiseError(message)

  def PrintErrorMessage(self, message):
    self._errorFile.write("{0:s}\n".format(message))
    self._errorFile.flush()

# 3) Access methods
# -----------------

  def CreateTools(self):
    self._scriptPrinter = S2MLScriptPrinter(self._engine)
    self._scriptParser = S2MLScriptParser(self._engine)
    self._scriptChecker = S2MLScriptChecker(self._engine)

  def GetScriptParser(self):
    return self._scriptParser

  def GetScriptPrinter(self):
    return self._scriptPrinter

  def GetScriptChecker(self):
    return self._scriptChecker

  def GetOutputFile(self):
    return self._outputFile

  def SetOutputFile(self, outputFile):
    self._outputFile = outputFile

  def GetErrorFile(self):
    return self._errorFile

  def SetErrorFile(self, errorFile):
    self._errorFile = errorFile

  def FlushOutputAndErrorFiles(self):
    self._outputFile.flush()
    self._errorFile.flush()

# 4) Scripts
# ----------

  def LoadScript(self, fileName):
    previousScript = self._currentScript
    self._currentScript = self._scriptParser.ImportScript(fileName)
    if self._currentScript==None:
      self._currentScript = previousScript
      return 1
    error = self._scriptChecker.CheckScript(self._currentScript)
    if error!=0:
      self._currentScript = previousScript
      return error
    error = self._engine.ExecuteScript(self._currentScript)
    self._currentScript = previousScript
    return error

